/* gIDE indent plugin
 * 
 * Run indent on the current document.
 * 
 * Copyright (C) 2001 Andre van Dijk "Unseen" <andre@unseen.demon.nl>
 * Copyright (C) 2001 Roberto Majadas "telemaco" <phoenix@nova.es>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.  
 */

#include <config.h>

#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <wait.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <liboaf/liboaf.h>
#include <libgide/libgide.h>
#include <gdl/GDL.h>

#define DEBUG
#define SAMPLE_COMPONENT_IID "OAFIID:GNOME_Development_Plugin:indent"
#define INDENT_SH_NAME "sh"
#define INDENT_SH_PATH "/bin/sh"

/* Predefined coding styles */
#define STYLE_CUSTOM 0
#define STYLE_GNU 1
#define STYLE_KR 2
#define STYLE_BERKELEY 3

typedef struct _indent_dialog indent_dialog;
typedef struct _indent_info indent_info;

struct _indent_dialog
{
  /* Options dialog */
  GtkWidget *dialog;
  /* Custom coding style radio option */
  GtkWidget *custom_radio;
  /* Force blank lines after the declarations. */
  GtkWidget *bad;
  /* Force blank lines after procedure bodies. */
  GtkWidget *bap;
  /* Force blank lines before block comments. */
  GtkWidget *bbb;
  /* Prefer to break long lines before boolean operators. */
  GtkWidget *bbo;
  /* Put braces on line after if, etc. */
  GtkWidget *bl;
  /* Force newline after comma in declaration. */
  GtkWidget *bc;
  /* Indent braces n spaces. */
  GtkWidget *bli;
  /* Put braces on the line after struct declaration lines. */
  GtkWidget *bls;
  /* Put braces on line with if, etc. */
  GtkWidget *br;
  /* Put braces on struct declaration line. */
  GtkWidget *brs;
  /* Put a space between sizeof and its argument */
  GtkWidget *bs;
  /* Put comments to the right of code in column n. */
  GtkWidget *c;
  /* Indent braces after a case label N spaces. */
  GtkWidget *cbi;
  /* Put comments to the right of the declarations in column n. */
  GtkWidget *cd;
  /* Put comment delimiters on blank lines. */
  GtkWidget *cdb;
  /* Cuddle while of do {} while; and preceeding `}. */
  GtkWidget *cdw;
  /* Cuddle while of do {} while; and preceeding `}. */
  GtkWidget *ce;
  /* Continuation indent of n spaces. */
  GtkWidget *ci;
  /* Case label indent of n spaces. */
  GtkWidget *cli;
  /* Put comments to the right of #else and #endif statements in column n. */
  GtkWidget *cp;
  /* Put a space after a cast operator. */
  GtkWidget *cs;
  /* Set indentation of comments not to the right of code to n spaces. */
  GtkWidget *d;
  /* Align all arguments in a declaration with opening paren. */
  GtkWidget *bfda;
  /* Put variables in column n. */
  GtkWidget *di;
  /* Format comments in the first column. */
  GtkWidget *fc1;
  /* Do not disable all formatting of comments. */
  GtkWidget *fca;
  /* Prefer to break long lines at the position of newlines in the input. */
  GtkWidget *hnl;
  /* Set indentation level to n spaces */
  GtkWidget *i;
  /* Indent parameter types in old-style function definitions by n spaces. */
  GtkWidget *ip;
  /* Set maximum line length for non-comment lines to n. */
  GtkWidget *l;
  /* Set maximum line length for comment formatting to n. */
  GtkWidget *lc;
  /* Line up continued lines at parentheses. */
  GtkWidget *lp;
  /* Leave space between `# and preprocessor directive. */
  GtkWidget *lps;
  /* Put space after the function in function calls. */
  GtkWidget *pcs;
  /* Specify the extra indentation per open parentheses ( * when a
     statement is broken. */
  GtkWidget *pi;
  /* Put a space after every ( and before every ). */
  GtkWidget *prs;
  /* Put the type of a procedure on the line before its name. */
  GtkWidget *psl;
  /* Put a space after each for. */
  GtkWidget *saf;
  /* Put a space after each if. */
  GtkWidget *sai;
  /* Put a space after each while. */
  GtkWidget *saw;
  /* Indent braces of a struct, union or enum N spaces. See STATEMENTS. */
  GtkWidget *sbi;
  /* Put the `* character at the left of comments. */
  GtkWidget *sc;
  /* Swallow optional blank lines. */
  GtkWidget *sob;
  /* On one-line for and while statments, force a blank before the semicolon. */
  GtkWidget *ss;
  /* Set tab size to n spaces. */
  GtkWidget *ts;
  /* Use tabs. This is the default. */
  GtkWidget *ut;
};

struct _indent_info
{
  gint coding_style;
  gchar *temp_file_name;
  GideTool *tool;
  gboolean setting_default;
};

static indent_info *indentinfo;
static indent_dialog *indentdialog;

/*
 * Get current document as a GString
 */
GString *indent_get_document (GideTool * tool);

/*
 * Execute indent on current document
 */
static void indent_execute ();

/*
 * Delete current document text and load temp file instead
 */
static void indent_load_temp_file ();

/*
 * Create dialog and show
 */
static void indent_create_dialog ();

/*
 * Close dialog and clean up
 */
static void indent_close_dialog (GtkWidget * widget, gpointer data);

/*
 * Create the notebook for the dialog
 */
static void indent_create_notebook ();

/* Dialog Callbacks */
void on_coding_gnu_toggled (GtkToggleButton * togglebutton,
			    gpointer user_data);
void on_coding_kr_toggled (GtkToggleButton * togglebutton,
			   gpointer user_data);
void on_coding_berkeley_toggled (GtkToggleButton * togglebutton,
				 gpointer user_data);
void on_coding_custom_toggled (GtkToggleButton * togglebutton,
			       gpointer user_data);
void on_custom_option_activated (GtkEditable * editable, gpointer user_data);

/*
 * Turn off all the options and set all the spin buttons to zero.
 */
static void indent_clear_options() {
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bad), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bap), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bbb), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bbo), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bc), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->hnl), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bl), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->br), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bls), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->brs), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bs), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cs), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->fca), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cdb), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cdw), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->ce), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->fc1), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->lp), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->lps), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bfda), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->pcs), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->prs), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->psl), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saf), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sai), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saw), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sc), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sob), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->ss), FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->ut), FALSE);

  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->l), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->lc), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->bli), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cbi), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cd), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cp), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->d), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->i), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ci), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->di), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->c), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cli), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->pi), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->sbi), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ts), 0);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ip), 0);
}

static void
indent_gnu_set_defaults ()
{
  indentinfo->setting_default=TRUE;
  indent_clear_options();
  /* -nbad */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bad), FALSE);
  /* -bap */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bap), TRUE);
  /* -nbc */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bc), FALSE);
  /* -bbo */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bbo), TRUE);
  /* -bl */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bl), TRUE);
  /* -bli2 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->bli), 2);
  /* -bls */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bls), TRUE);
  /* -ncdb */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cdb), FALSE);
  /* -cp1 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cp), 2);
  /* -cs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cs), TRUE);
  /* -di2 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->di), 2);
  /* -ndj ?????? */
  /* -nfc1 */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->fc1), FALSE);
  /* -nfca */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->fca), FALSE);
  /* -hnl */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->hnl), TRUE);
  /* -i2 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->i), 2);
  /* -lp */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->lp), TRUE);
  /* -pcs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->pcs), TRUE);
  /* -psl */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->psl), TRUE);
  /* -saf */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saf), TRUE);
  /* -sai */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sai), TRUE);
  /* -saw */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saw), TRUE);
  /* -nsc */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sc), FALSE);
  /* -nsob */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sob), FALSE);
  /* Don't suppose this would be thread safe ... */
  indentinfo->setting_default=FALSE;
}

static void
indent_kr_set_defaults ()
{
  indentinfo->setting_default=TRUE;
  indent_clear_options();
  /* -nbad */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bad), FALSE);
  /* -bap */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bap), TRUE);
  /* -nbc */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bc), FALSE);
  /* -bbo */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bbo), TRUE);
  /* -br */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->br), TRUE);
  /* -brs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->brs), TRUE);
  /* -c33 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->c), 33);
  /* -cd33 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cd), 33);
  /* -ncdb */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cdb), FALSE);
  /* -ce */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->ce), TRUE);
  /* -ci4 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ci), 4);
  /* -cli0 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cli), 0);
  /* -cp33 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cp), 33);
  /* -cs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cs), TRUE);
  /* -d0 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->d), 0);
  /* -di1 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->di), 1);
  /* -nfc1 */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->fc1), FALSE);
  /* -nfca */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->fca), FALSE);
  /* -hnl */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->hnl), TRUE);
  /* -i4 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->i), 4);
  /* -ip0 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ip), 0);
  /* -l75 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->l), 75);
  /* -lp */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->lp), TRUE);
  /* -npcs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->pcs), FALSE);
  /* -nprs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->prs), FALSE);
  /* -npsl */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->psl), FALSE);
  /* -saf */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saf), TRUE);
  /* -sai */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sai), TRUE);
  /* -saw */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saw), TRUE);
  /* -nsc */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sc), FALSE);
  /* -nsob */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sob), FALSE);
  /* -nss */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->ss), FALSE);
  indentinfo->setting_default=FALSE;
}

static void
indent_berkeley_set_defaults ()
{
  indentinfo->setting_default=TRUE;
  indent_clear_options();
  /* -nbad */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bad), FALSE);
  /* -bap */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bap), TRUE);
  /* -bbo */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->bbo), TRUE);
  /* -br */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->br), TRUE);
  /* -brs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->brs), TRUE);
  /* -c33 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->c), 33);
  /* -cd33 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cd), 33);
  /* -cdb */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->cdb), TRUE);
  /* -ce */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->ce), TRUE);
  /* -ci4 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ci), 4);
  /* -cli0 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cli), 0);
  /* -cp33 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->cp), 33);
  /* -di16 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->di), 16);
  /* -fc1 */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->fc1), TRUE);
  /* -hnl */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->hnl), TRUE);
  /* -i4 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->i), 4);
  /* -ip4 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ip), 4);
  /* -ip0 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ip), 0);
  /* -l75 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->l), 75);
  /* -lp */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->lp), TRUE);
  /* -npcs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->pcs), FALSE);
  /* -nprs */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->prs), FALSE);
  /* -psl */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->psl), TRUE);
  /* -saf */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saf), TRUE);
  /* -sai */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sai), TRUE);
  /* -saw */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->saw), TRUE);
  /* -sc */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sc), TRUE);
  /* -nsob */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->sob), FALSE);
  /* -nss */
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (indentdialog->ss), FALSE);
  /* -ts8 */
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (indentdialog->ts), 8);
  indentinfo->setting_default=FALSE;
}

void
on_coding_gnu_toggled (GtkToggleButton * togglebutton, gpointer user_data)
{
#ifdef DEBUG
  g_print ("GNU coding style selected\n");
#endif
  /* Set GNU defaults */
  indent_gnu_set_defaults ();
  indentinfo->coding_style = STYLE_GNU;
}


void
on_coding_kr_toggled (GtkToggleButton * togglebutton, gpointer user_data)
{
#ifdef DEBUG
  g_print ("K&R coding style selected\n");
#endif
  indent_kr_set_defaults ();
  indentinfo->coding_style = STYLE_KR;
}


void
on_coding_berkeley_toggled (GtkToggleButton * togglebutton,
			    gpointer user_data)
{
#ifdef DEBUG
  g_print ("Berkeley coding style selected\n");
#endif
  indent_berkeley_set_defaults ();
  indentinfo->coding_style = STYLE_BERKELEY;
}

void
on_coding_custom_toggled (GtkToggleButton * togglebutton, gpointer user_data)
{
#ifdef DEBUG
  g_print ("Custom coding style selected\n");
#endif
  indentinfo->coding_style = STYLE_CUSTOM;
}

void
on_custom_option_activated (GtkEditable * editable, gpointer user_data)
{
  /* Toggle custom coding style when an indent option is changed */
  if (!indentinfo->setting_default)
		 /* && !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				    (indentdialog->custom_radio)))
    {*/
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				    (indentdialog->custom_radio), TRUE);
//    }
}

static void
indent_close_dialog (GtkWidget * widget, gpointer data)
{
  gnome_dialog_close (GNOME_DIALOG (indentdialog->dialog));
  g_free (indentinfo->temp_file_name);
  g_free (indentinfo);
  g_free (indentdialog);
}

static void
indent_load_temp_file ()
{
  gint fd, i;
  gint pos = 0;
  gchar buf[1024];

  if ((fd = open (indentinfo->temp_file_name, O_RDONLY)) == -1)
    {
      gnome_error_dialog ("Error opening temporary file.");
      return;
    }

  /* Delete current document text */
  gide_delete_text (indentinfo->tool, 0,
		    gide_get_document_length (indentinfo->tool));

  /* Read temporary file into buffer */
  while ((i = read (fd, buf, 1023)) > 0)
    {
      buf[i] = '\0';
      gide_insert_text_at_pos (indentinfo->tool, pos, buf);
      pos += i;
    }
}

static gchar *
indent_get_arg_str (void)
{
  GString *coding_arg;

  coding_arg = g_string_new ("");
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bad)))
    {
      g_string_append (coding_arg, _(" -bad"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nbad"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bap)))
    {
      g_string_append (coding_arg, _(" -bap"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nbap"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bbb)))
    {
      g_string_append (coding_arg, _(" -bbb"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nbbb"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bbo)))
    {
      g_string_append (coding_arg, _(" -bbo"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nbbo"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bc)))
    {
      g_string_append (coding_arg, _(" -bc"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nbc"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->hnl)))
    {
      g_string_append (coding_arg, _(" -hnl"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nhnl"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bl)))
    {
      g_string_append (coding_arg, _(" -bl"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->br)))
    {
      g_string_append (coding_arg, _(" -br"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bls)))
    {
      g_string_append (coding_arg, _(" -bls"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->brs)))
    {
      g_string_append (coding_arg, _(" -brs"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bs)))
    {
      g_string_append (coding_arg, _(" -bs"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->cs)))
    {
      g_string_append (coding_arg, _(" -cs"));
    }
  else
    {
      g_string_append (coding_arg, _(" -ncs"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->fca)))
    {
      g_string_append (coding_arg, _(" -fca"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nfca"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->cdb)))
    {
      g_string_append (coding_arg, _(" -cdb"));
    }
  else
    {
      g_string_append (coding_arg, _(" -ncdb"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->cdw)))
    {
      g_string_append (coding_arg, _(" -cdw"));
    }
  else
    {
      g_string_append (coding_arg, _(" -ncdw"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->ce)))
    {
      g_string_append (coding_arg, _(" -ce"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nce"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->fc1)))
    {
      g_string_append (coding_arg, _(" -fc1"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nfc1"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->lp)))
    {
      g_string_append (coding_arg, _(" -lp"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nlp"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->lps)))
    {
      g_string_append (coding_arg, _(" -lps"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->bfda)))
    {
      g_string_append (coding_arg, _(" -bfda"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nbfda"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->pcs)))
    {
      g_string_append (coding_arg, _(" -pcs"));
    }
  else
    {
      g_string_append (coding_arg, _(" -npcs"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->prs)))
    {
      g_string_append (coding_arg, _(" -prs"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nprs"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->psl)))
    {
      g_string_append (coding_arg, _(" -psl"));
    }
  else
    {
      g_string_append (coding_arg, _(" -npsl"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->saf)))
    {
      g_string_append (coding_arg, _(" -saf"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nsaf"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->sai)))
    {
      g_string_append (coding_arg, _(" -sai"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nsai"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->saw)))
    {
      g_string_append (coding_arg, _(" -saw"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nsaw"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->sc)))
    {
      g_string_append (coding_arg, _(" -sc"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nsc"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->sob)))
    {
      g_string_append (coding_arg, _(" -sob"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nsob"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->ss)))
    {
      g_string_append (coding_arg, _(" -ss"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nss"));
    }
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (indentdialog->ut)))
    {
      g_string_append (coding_arg, _(" -ut"));
    }
  else
    {
      g_string_append (coding_arg, _(" -nut"));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->l))
      > 0)
    {
      g_string_sprintfa (coding_arg, " -l%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    l)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->lc)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -lc%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    lc)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->bli)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -bli%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    bli)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->cbi)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -cbi%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    cbi)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->cd)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -cd%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    cd)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->cp)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -cp%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    cp)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->d)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -d%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    d)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->i)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -i%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    i)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->ci)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -ci%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    ci)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->di)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -di%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    di)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->c)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -c%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    c)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->cli)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -cli%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    cli)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->pi)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -pi%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    pi)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->sbi)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -sbi%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    sbi)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->ts)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -ts%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    ts)));
    }
  if (gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (indentdialog->ip)) >
      0)
    {
      g_string_sprintfa (coding_arg, " -ip%d",
			 gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON
							   (indentdialog->
							    ip)));
    }
  else
    {
      /* Zero width identation for parameters */
      g_string_append (coding_arg, _(" -nip"));
    }

#ifdef DEBUG
  g_print ("Options selected: %s\n", coding_arg->str);
#endif
  if (coding_arg->len > 0)
    return coding_arg->str;
  else
    return NULL;
}

static void
indent_execute ()
{
  gint leftpipe[2];
  gint fd;
  gint pid;
  gchar *arg[4];
  gchar *command_to_run;
  gchar *coding_style_arg;
  GString *buffer_to_pipe;

#ifdef DEBUG
  g_print ("indent_execute\n");
#endif

  /* Create pipe */
  if (pipe (leftpipe) == -1)
    {
      gnome_error_dialog
	("Unable to create communication channel for indent.");
      indent_close_dialog (NULL, NULL);
      return;
    }

  /* Get current document text */
  buffer_to_pipe = indent_get_document (indentinfo->tool);
  if (buffer_to_pipe == NULL)
    {
      /* No document so not much to do */
      indent_close_dialog (NULL, NULL);
      return;
    }

  /* Construct command string with options */
  switch (indentinfo->coding_style)
    {
    case STYLE_GNU:
      coding_style_arg = g_strdup(" -gnu");
      break;
    case STYLE_KR:
      coding_style_arg = g_strdup(" -kr");
      break;
    case STYLE_BERKELEY:
      coding_style_arg = g_strdup(" -orig");
      break;
    default:
      coding_style_arg = indent_get_arg_str ();
      break;
    }
  command_to_run = g_strdup_printf ("indent -st%s", coding_style_arg);
  g_free (coding_style_arg);
#ifdef DEBUG
  g_print ("indent_execute: command to run: %s\n", command_to_run);
#endif

  pid = fork ();
  if (pid == 0)
    {				/* Child process */

      /* Stdin reads from left pipe */
      dup2 (leftpipe[0], 0);
      close (leftpipe[0]);
      close (leftpipe[1]);

      /* Stdout writes to temp file */
      fd = open (indentinfo->temp_file_name, O_RDWR | O_CREAT, S_IRWXU);
      if (fd == -1)
	{
	  g_warning ("Unable to create temporary file\n");
	  return;
	}
      dup2 (fd, 1);
      close (fd);

      /* Run shell with indent */
      arg[0] = INDENT_SH_NAME;
      arg[1] = "-c";
      arg[2] = command_to_run;
      arg[3] = NULL;

      execvp (INDENT_SH_PATH, arg);

      g_warning ("An undetermined PIPE problem occurred\n");
      return;
    }
  else if (pid == -1)
    {
      gnome_error_dialog ("Problem running indent.");
    }
  else
    {
      close (leftpipe[0]);
#ifdef DEBUG
      g_print ("Writing %d bytes\n", buffer_to_pipe->len);
#endif

      /* Write document to pipe */
      if (buffer_to_pipe->len > 0)
	{
	  fcntl (leftpipe[1], F_SETFD, FD_CLOEXEC);
	  write (leftpipe[1], buffer_to_pipe->str, buffer_to_pipe->len);
	  close (leftpipe[1]);
	}
      waitpid (pid, NULL, 0);
      indent_load_temp_file ();
    }
  g_string_free (buffer_to_pipe, TRUE);
  g_free (command_to_run);
  indent_close_dialog (NULL, NULL);
}

static void
create_custom_tabs (GtkWidget * notebook1)
{
  GtkTooltips *button_bar_tips;
  GtkWidget *label19;
  GtkWidget *blanksVbox;
  GtkWidget *table1;
  GtkWidget *bbb;
  GtkWidget *bad;
  GtkWidget *bap;
  GtkWidget *bbo;
  GtkWidget *hbox13;
  GtkWidget *label13;
  GtkObject *l_adj;
  GtkWidget *l;
  GtkWidget *sob;
  GtkWidget *hnl;
  GtkWidget *blanksTab;
  GtkWidget *statementsVbox;
  GtkWidget *table2;
  GtkWidget *bl;
  GtkWidget *hbox3;
  GtkWidget *label3;
  GtkObject *bli_adj;
  GtkWidget *bli;
  GtkWidget *br;
  GtkWidget *hbox7;
  GtkWidget *label7;
  GtkObject *ci_adj;
  GtkWidget *ci;
  GtkWidget *ce;
  GtkWidget *cdw;
  GtkWidget *hbox5;
  GtkWidget *label5;
  GtkObject *cbi_adj;
  GtkWidget *cbi;
  GtkWidget *cs;
  GtkWidget *hbox16;
  GtkWidget *label20;
  GtkObject *pi_adj;
  GtkWidget *pi;
  GtkWidget *pcs;
  GtkWidget *saf;
  GtkWidget *saw;
  GtkWidget *sai;
  GtkWidget *bs;
  GtkWidget *hbox15;
  GtkObject *cli_adj;
  GtkWidget *cli;
  GtkWidget *hbox17;
  GtkWidget *label21;
  GtkObject *sbi_adj;
  GtkWidget *sbi;
  GtkWidget *ss;
  GtkWidget *prs;
  GtkWidget *psl;
  GtkWidget *statementsTab;
  GtkWidget *commentsVbox;
  GtkWidget *table3;
  GtkWidget *hbox6;
  GtkWidget *label6;
  GtkObject *cd_adj;
  GtkWidget *cd;
  GtkWidget *hbox4;
  GtkWidget *label4;
  GtkObject *c_adj;
  GtkWidget *c;
  GtkWidget *cdb;
  GtkWidget *hbox8;
  GtkWidget *label8;
  GtkObject *cp_adj;
  GtkWidget *cp;
  GtkWidget *hbox9;
  GtkWidget *label9;
  GtkObject *d_adj;
  GtkWidget *d;
  GtkWidget *fc1;
  GtkWidget *fca;
  GtkWidget *hbox14;
  GtkWidget *label14;
  GtkObject *lc_adj;
  GtkWidget *lc;
  GtkWidget *sc;
  GtkWidget *CommentsTab;
  GtkWidget *indentVbox;
  GtkWidget *table5;
  GtkWidget *hbox12;
  GtkWidget *label12;
  GtkObject *ip_adj;
  GtkWidget *ip;
  GtkWidget *hbox11;
  GtkWidget *label11;
  GtkObject *i_adj;
  GtkWidget *i;
  GtkWidget *hbox18;
  GtkWidget *label22;
  GtkObject *ts_adj;
  GtkWidget *ts;
  GtkWidget *lp;
  GtkWidget *ut;
  GtkWidget *lps;
  GtkWidget *indentationsTab;
  GtkWidget *declarationsVbox;
  GtkWidget *table4;
  GtkWidget *hbox10;
  GtkWidget *label10;
  GtkObject *di_adj;
  GtkWidget *di;
  GtkWidget *bc;
  GtkWidget *brs;
  GtkWidget *bfda;
  GtkWidget *bls;
  GtkWidget *declarationsTab;

  blanksVbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (blanksVbox);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "blanksVbox", blanksVbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (blanksVbox);
  gtk_container_add (GTK_CONTAINER (notebook1), blanksVbox);

  table1 = gtk_table_new (4, 2, TRUE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (blanksVbox), table1, FALSE, TRUE, 0);

  bbb = gtk_check_button_new_with_label (_("Blank lines before block comments"));
  gtk_widget_ref (bbb);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bbb", bbb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bbb);
  gtk_table_attach (GTK_TABLE (table1), bbb, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  bad = gtk_check_button_new_with_label (_("Force blank lines after declarations"));
  gtk_widget_ref (bad);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bad", bad,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bad);
  gtk_table_attach (GTK_TABLE (table1), bad, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  bap = gtk_check_button_new_with_label (_("Force blank lines after procedure bodies"));
  gtk_widget_ref (bap);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bap", bap,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bap);
  gtk_table_attach (GTK_TABLE (table1), bap, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  bbo = gtk_check_button_new_with_label (_("Break long lines before boolean operators"));
  gtk_widget_ref (bbo);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bbo", bbo,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bbo);
  gtk_table_attach (GTK_TABLE (table1), bbo, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox13);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox13", hbox13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox13);
  gtk_table_attach (GTK_TABLE (table1), hbox13, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label13 = gtk_label_new (_("Maximum line length:"));
  gtk_widget_ref (label13);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label13", label13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label13);
  gtk_box_pack_start (GTK_BOX (hbox13), label13, FALSE, FALSE, 0);

  l_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  l = gtk_spin_button_new (GTK_ADJUSTMENT (l_adj), 1, 0);
  gtk_widget_ref (l);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "l", l,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (l);
  gtk_box_pack_start (GTK_BOX (hbox13), l, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (l), TRUE);

  sob = gtk_check_button_new_with_label (_("Swallow option blank lines"));
  gtk_widget_ref (sob);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "sob", sob,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sob);
  gtk_table_attach (GTK_TABLE (table1), sob, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hnl = gtk_check_button_new_with_label (_("Prefer to break lines at newlines in input"));
  gtk_widget_ref (hnl);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hnl", hnl,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hnl);
  gtk_table_attach (GTK_TABLE (table1), hnl, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  blanksTab = gtk_label_new (_("Blank lines & line breaking"));
  gtk_widget_ref (blanksTab);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "blanksTab", blanksTab,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (blanksTab);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), blanksTab);

  statementsVbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (statementsVbox);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "statementsVbox", statementsVbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statementsVbox);
  gtk_container_add (GTK_CONTAINER (notebook1), statementsVbox);

  table2 = gtk_table_new (10, 2, TRUE);
  gtk_widget_ref (table2);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "table2", table2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (statementsVbox), table2, FALSE, TRUE, 0);

  bl = gtk_check_button_new_with_label (_("Put braces on line after if etc."));
  gtk_widget_ref (bl);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bl", bl,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bl);
  gtk_table_attach (GTK_TABLE (table2), bl, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_table_attach (GTK_TABLE (table2), hbox3, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label3 = gtk_label_new (_("Brace indent:"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox3), label3, FALSE, FALSE, 0);

  bli_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  bli = gtk_spin_button_new (GTK_ADJUSTMENT (bli_adj), 1, 0);
  gtk_widget_ref (bli);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bli", bli,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bli);
  gtk_box_pack_start (GTK_BOX (hbox3), bli, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (bli), TRUE);

  br = gtk_check_button_new_with_label (_("Put braces on line with if etc."));
  gtk_widget_ref (br);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "br", br,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (br);
  gtk_table_attach (GTK_TABLE (table2), br, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox7", hbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7);
  gtk_table_attach (GTK_TABLE (table2), hbox7, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label7 = gtk_label_new (_("Continuation indentation:"));
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_box_pack_start (GTK_BOX (hbox7), label7, FALSE, FALSE, 0);

  ci_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  ci = gtk_spin_button_new (GTK_ADJUSTMENT (ci_adj), 1, 0);
  gtk_widget_ref (ci);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "ci", ci,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ci);
  gtk_box_pack_start (GTK_BOX (hbox7), ci, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ci), TRUE);

  ce = gtk_check_button_new_with_label (_("Cuddle else"));
  gtk_widget_ref (ce);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "ce", ce,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ce);
  gtk_table_attach (GTK_TABLE (table2), ce, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  cdw = gtk_check_button_new_with_label (_("Cuddle do while"));
  gtk_widget_ref (cdw);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "cdw", cdw,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cdw);
  gtk_table_attach (GTK_TABLE (table2), cdw, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox5 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox5);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox5", hbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox5);
  gtk_table_attach (GTK_TABLE (table2), hbox5, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND), 0, 0);

  label5 = gtk_label_new (_("Indent braces after case label: "));
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox5), label5, FALSE, FALSE, 0);

  cbi_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  cbi = gtk_spin_button_new (GTK_ADJUSTMENT (cbi_adj), 1, 0);
  gtk_widget_ref (cbi);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "cbi", cbi,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cbi);
  gtk_box_pack_start (GTK_BOX (hbox5), cbi, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (cbi), TRUE);

  cs = gtk_check_button_new_with_label (_("Space after cast"));
  gtk_widget_ref (cs);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "cs", cs,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cs);
  gtk_table_attach (GTK_TABLE (table2), cs, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox16 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox16);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox16", hbox16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox16);
  gtk_table_attach (GTK_TABLE (table2), hbox16, 1, 2, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label20 = gtk_label_new (_("Extra indent per open parentheses:"));
  gtk_widget_ref (label20);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label20", label20,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label20);
  gtk_box_pack_start (GTK_BOX (hbox16), label20, FALSE, FALSE, 0);

  pi_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  pi = gtk_spin_button_new (GTK_ADJUSTMENT (pi_adj), 1, 0);
  gtk_widget_ref (pi);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "pi", pi,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pi);
  gtk_box_pack_start (GTK_BOX (hbox16), pi, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (pi), TRUE);

  pcs = gtk_check_button_new_with_label (_("Space between procedure name and ("));
  gtk_widget_ref (pcs);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "pcs", pcs,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (pcs);
  gtk_table_attach (GTK_TABLE (table2), pcs, 0, 1, 5, 6,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  saf = gtk_check_button_new_with_label (_("Put a space after each for"));
  gtk_widget_ref (saf);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "saf", saf,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (saf);
  gtk_table_attach (GTK_TABLE (table2), saf, 0, 1, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  saw = gtk_check_button_new_with_label (_("Put a space after each while"));
  gtk_widget_ref (saw);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "saw", saw,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (saw);
  gtk_table_attach (GTK_TABLE (table2), saw, 0, 1, 8, 9,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  sai = gtk_check_button_new_with_label (_("Put a space after each if"));
  gtk_widget_ref (sai);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "sai", sai,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sai);
  gtk_table_attach (GTK_TABLE (table2), sai, 1, 2, 7, 8,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  bs = gtk_check_button_new_with_label (_("Space between sizoef and argument"));
  gtk_widget_ref (bs);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bs", bs,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bs);
  gtk_table_attach (GTK_TABLE (table2), bs, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox15 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox15);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox15", hbox15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox15);
  gtk_table_attach (GTK_TABLE (table2), hbox15, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label19 = gtk_label_new (_("Case label indentation: "));
  gtk_widget_ref (label19);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label19", label19,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (hbox15), label19, FALSE, FALSE, 0);

  cli_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  cli = gtk_spin_button_new (GTK_ADJUSTMENT (cli_adj), 1, 0);
  gtk_widget_ref (cli);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "cli", cli,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cli);
  gtk_box_pack_start (GTK_BOX (hbox15), cli, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (cli), TRUE);

  hbox17 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox17);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox17", hbox17,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox17);
  gtk_table_attach (GTK_TABLE (table2), hbox17, 1, 2, 8, 9,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label21 = gtk_label_new (_("Indent for braces of a struct, union or enum: "));
  gtk_widget_ref (label21);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label21", label21,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label21);
  gtk_box_pack_start (GTK_BOX (hbox17), label21, FALSE, FALSE, 0);

  sbi_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  sbi = gtk_spin_button_new (GTK_ADJUSTMENT (sbi_adj), 1, 0);
  gtk_widget_ref (sbi);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "sbi", sbi,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sbi);
  gtk_box_pack_start (GTK_BOX (hbox17), sbi, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (sbi), TRUE);

  ss = gtk_check_button_new_with_label (_("Space special colon"));
  gtk_widget_ref (ss);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "ss", ss,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ss);
  gtk_table_attach (GTK_TABLE (table2), ss, 0, 1, 9, 10,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  prs = gtk_check_button_new_with_label (_("Put space after every ( and before every )"));
  gtk_widget_ref (prs);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "prs", prs,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (prs);
  gtk_table_attach (GTK_TABLE (table2), prs, 0, 1, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  psl = gtk_check_button_new_with_label (_("Put type of procedure on previous line"));
  gtk_widget_ref (psl);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "psl", psl,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (psl);
  gtk_table_attach (GTK_TABLE (table2), psl, 1, 2, 6, 7,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  statementsTab = gtk_label_new (_("Statements"));
  gtk_widget_ref (statementsTab);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "statementsTab", statementsTab,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statementsTab);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), statementsTab);

  commentsVbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (commentsVbox);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "commentsVbox", commentsVbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (commentsVbox);
  gtk_container_add (GTK_CONTAINER (notebook1), commentsVbox);

  table3 = gtk_table_new (5, 2, TRUE);
  gtk_widget_ref (table3);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "table3", table3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (commentsVbox), table3, FALSE, TRUE, 0);

  hbox6 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox6);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox6", hbox6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox6);
  gtk_table_attach (GTK_TABLE (table3), hbox6, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label6 = gtk_label_new (_("Declaration comment column:"));
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox6), label6, FALSE, FALSE, 0);

  cd_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  cd = gtk_spin_button_new (GTK_ADJUSTMENT (cd_adj), 1, 0);
  gtk_widget_ref (cd);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "cd", cd,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cd);
  gtk_box_pack_start (GTK_BOX (hbox6), cd, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (cd), TRUE);

  hbox4 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox4);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox4", hbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox4);
  gtk_table_attach (GTK_TABLE (table3), hbox4, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label4 = gtk_label_new (_("Comment indentation:"));
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox4), label4, FALSE, FALSE, 0);

  c_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  c = gtk_spin_button_new (GTK_ADJUSTMENT (c_adj), 1, 0);
  gtk_widget_ref (c);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "c", c,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (c);
  gtk_box_pack_start (GTK_BOX (hbox4), c, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (c), TRUE);

  cdb = gtk_check_button_new_with_label (_("Comment delimiters on blank lines"));
  gtk_widget_ref (cdb);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "cdb", cdb,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cdb);
  gtk_table_attach (GTK_TABLE (table3), cdb, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox8 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox8);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox8", hbox8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox8);
  gtk_table_attach (GTK_TABLE (table3), hbox8, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label8 = gtk_label_new (_("Comment right of #else in column:"));
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_box_pack_start (GTK_BOX (hbox8), label8, FALSE, FALSE, 0);

  cp_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  cp = gtk_spin_button_new (GTK_ADJUSTMENT (cp_adj), 1, 0);
  gtk_widget_ref (cp);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "cp", cp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (cp);
  gtk_box_pack_start (GTK_BOX (hbox8), cp, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (cp), TRUE);

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox9);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox9", hbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox9);
  gtk_table_attach (GTK_TABLE (table3), hbox9, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  label9 = gtk_label_new (_("Comments not right of code at column:"));
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_box_pack_start (GTK_BOX (hbox9), label9, FALSE, FALSE, 0);

  d_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  d = gtk_spin_button_new (GTK_ADJUSTMENT (d_adj), 1, 0);
  gtk_widget_ref (d);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "d", d,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (d);
  gtk_box_pack_start (GTK_BOX (hbox9), d, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (d), TRUE);

  fc1 = gtk_check_button_new_with_label (_("Format first column comments"));
  gtk_widget_ref (fc1);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "fc1", fc1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fc1);
  gtk_table_attach (GTK_TABLE (table3), fc1, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  fca = gtk_check_button_new_with_label (_("Format all comments"));
  gtk_widget_ref (fca);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "fca", fca,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fca);
  gtk_table_attach (GTK_TABLE (table3), fca, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  hbox14 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox14);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox14", hbox14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox14);
  gtk_table_attach (GTK_TABLE (table3), hbox14, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label14 = gtk_label_new (_("Comment line length:"));
  gtk_widget_ref (label14);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label14", label14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label14);
  gtk_box_pack_start (GTK_BOX (hbox14), label14, FALSE, FALSE, 0);

  lc_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  lc = gtk_spin_button_new (GTK_ADJUSTMENT (lc_adj), 1, 0);
  gtk_widget_ref (lc);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "lc", lc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lc);
  gtk_box_pack_start (GTK_BOX (hbox14), lc, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (lc), TRUE);

  sc = gtk_check_button_new_with_label (_("Put the * character at the left of comments"));
  gtk_widget_ref (sc);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "sc", sc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (sc);
  gtk_table_attach (GTK_TABLE (table3), sc, 0, 1, 4, 5,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  CommentsTab = gtk_label_new (_("Comments"));
  gtk_widget_ref (CommentsTab);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "CommentsTab", CommentsTab,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (CommentsTab);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), CommentsTab);

  indentVbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (indentVbox);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "indentVbox", indentVbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (indentVbox);
  gtk_container_add (GTK_CONTAINER (notebook1), indentVbox);

  table5 = gtk_table_new (3, 2, TRUE);
  gtk_widget_ref (table5);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "table5", table5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table5);
  gtk_box_pack_start (GTK_BOX (indentVbox), table5, FALSE, TRUE, 0);

  hbox12 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox12);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox12", hbox12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox12);
  gtk_table_attach (GTK_TABLE (table5), hbox12, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label12 = gtk_label_new (_("Parameter indentation:"));
  gtk_widget_ref (label12);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label12", label12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label12);
  gtk_box_pack_start (GTK_BOX (hbox12), label12, FALSE, FALSE, 0);

  ip_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  ip = gtk_spin_button_new (GTK_ADJUSTMENT (ip_adj), 1, 0);
  gtk_widget_ref (ip);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "ip", ip,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ip);
  gtk_box_pack_start (GTK_BOX (hbox12), ip, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ip), TRUE);

  hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox11);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox11", hbox11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox11);
  gtk_table_attach (GTK_TABLE (table5), hbox11, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  label11 = gtk_label_new (_("Indent level:"));
  gtk_widget_ref (label11);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label11", label11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label11);
  gtk_box_pack_start (GTK_BOX (hbox11), label11, FALSE, FALSE, 0);

  i_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  i = gtk_spin_button_new (GTK_ADJUSTMENT (i_adj), 1, 0);
  gtk_widget_ref (i);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "i", i,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (i);
  gtk_box_pack_start (GTK_BOX (hbox11), i, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (i), TRUE);

  hbox18 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox18);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox18", hbox18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox18);
  gtk_table_attach (GTK_TABLE (table5), hbox18, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  label22 = gtk_label_new (_("Tab size:"));
  gtk_widget_ref (label22);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label22", label22,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label22);
  gtk_box_pack_start (GTK_BOX (hbox18), label22, FALSE, FALSE, 0);

  ts_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  ts = gtk_spin_button_new (GTK_ADJUSTMENT (ts_adj), 1, 0);
  gtk_widget_ref (ts);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "ts", ts,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ts);
  gtk_box_pack_start (GTK_BOX (hbox18), ts, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ts), TRUE);

  lp = gtk_check_button_new_with_label (_("Line up continued lines at parentheses"));
  gtk_widget_ref (lp);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "lp", lp,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lp);
  gtk_table_attach (GTK_TABLE (table5), lp, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  ut = gtk_check_button_new_with_label (_("Use tabs"));
  gtk_widget_ref (ut);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "ut", ut,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (ut);
  gtk_table_attach (GTK_TABLE (table5), ut, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  lps = gtk_check_button_new_with_label (_("Leave space # and preprocessor directive"));
  gtk_widget_ref (lps);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "lps", lps,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (lps);
  gtk_table_attach (GTK_TABLE (table5), lps, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  indentationsTab = gtk_label_new (_("Indentations"));
  gtk_widget_ref (indentationsTab);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "indentationsTab", indentationsTab,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (indentationsTab);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 4), indentationsTab);

  declarationsVbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (declarationsVbox);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "declarationsVbox", declarationsVbox,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (declarationsVbox);
  gtk_container_add (GTK_CONTAINER (notebook1), declarationsVbox);

  table4 = gtk_table_new (3, 2, TRUE);
  gtk_widget_ref (table4);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "table4", table4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table4);
  gtk_box_pack_start (GTK_BOX (declarationsVbox), table4, FALSE, TRUE, 0);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_table_attach (GTK_TABLE (table4), hbox10, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);

  label10 = gtk_label_new (_("Declaration indentation:"));
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_box_pack_start (GTK_BOX (hbox10), label10, FALSE, FALSE, 0);

  di_adj = gtk_adjustment_new (0, 0, 100, 1, 10, 10);
  di = gtk_spin_button_new (GTK_ADJUSTMENT (di_adj), 1, 0);
  gtk_widget_ref (di);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "di", di,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (di);
  gtk_box_pack_start (GTK_BOX (hbox10), di, FALSE, FALSE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (di), TRUE);

  bc = gtk_check_button_new_with_label (_("Force new line after comma in declaration"));
  gtk_widget_ref (bc);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bc", bc,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bc);
  gtk_table_attach (GTK_TABLE (table4), bc, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  brs = gtk_check_button_new_with_label (_("Put braces on line with struct decl. line"));
  gtk_widget_ref (brs);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "brs", brs,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (brs);
  gtk_table_attach (GTK_TABLE (table4), brs, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  bfda = gtk_check_button_new_with_label (_("Align arguments with opening parenthesis"));
  gtk_widget_ref (bfda);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bfda", bfda,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bfda);
  gtk_table_attach (GTK_TABLE (table4), bfda, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  bls = gtk_check_button_new_with_label (_("Put braces on line after struct decl. line"));
  gtk_widget_ref (bls);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "bls", bls,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (bls);
  gtk_table_attach (GTK_TABLE (table4), bls, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  declarationsTab = gtk_label_new (_("Declarations"));
  gtk_widget_ref (declarationsTab);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "declarationsTab", declarationsTab,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (declarationsTab);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 5), declarationsTab);

  /* Tooltips */
  button_bar_tips = gtk_tooltips_new();
  /* Add the tips */
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bad,
				 "This option causes indent to force a blank line after every block of declarations.","");
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bap,
				 "This option causes indent to force a blank line after every procedure body.",""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bbb,
				 "This option causes indent to force a blank line before every boxed comment",""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bbo,
				 "This option causes indent to prefer to break long lines before the boolean operators && and ||.",""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bl,
				 "Put braces on line after if, etc." ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bc,
				 "This options causes indent to force a newline after each comma in a declaration" ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bli,
				 "This option specifies the number of spaces by which braces are indented" ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bls,
				 "Put braces on the line after struct declaration lines."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), br,
				 "Put braces on line with if, etc." ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), brs,
				 "Put braces on struct declaration line."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bs,
				 "This option ensures that there is a space between the keyword sizeof and its argument."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), c,
				 "This option specifies the column for comments following code."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), cbi,
				 "The indentation of the braces below a case statement can be controlled with this option."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), cd,
				 "This option specifies the column for comments following declarations"
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), cdb,
				 "This option places the comment delimiters  on  blank lines."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), cdw,
				 "This option puts while and the preceding } on the same line."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), ce,
				 "This option puts else and the preceding } on the same line."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), ci,
				 "When a statement is broken across two lines, the second line is indented by a number of additional spaces specified by this option."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), cp,
				 "This option specifies the column for comments following preprocessor directives #else and #endif."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), cs,
				 "This option causes indent to put a space after a cast operator."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), cli,
				 "This option specifies the number of spaces that case labels should be indented to the right of  the  containing switch statement."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), d,
				 "If the comment begins a line (i.e., there is no program text to its left), it will be indented to the column it was found in unless the comment is within a block of code. In that case, such a comment will be aligned with the indented code of that block (unless the comment began in the  first column). This alignment may be affected by this option, which specifies an amount by which such comments are moved to the left, or unindented."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), bfda,
				 "This option causes a newline to be forced after the comma separating the arguments of a function declaration. The arguments will appear at the current indention level matching the opening paren. This is particularly helpful for functions with long argument lists."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), di,
				 "Indent will line up identifiers, in the column specified by this option"
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), fc1,
				 "To format comments beginning in column one, specify this option"
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), fca,
				 "Formatting of comments which begin after the first column is enabled with the this option."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), hnl,
				 "Prefer to break long lines at the position of newlines in the input."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), i,
				 "One issue in the formatting of code is how far each line should be indented from the left margin. When the beginning of a statement such as if or for is encountered, the indentation level is increased by the value specified by this option."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), ip,
				 "The indentation of type declarations in old-style function definitions is controlled by this option."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), l,
				 "Set maximum line length for non-comment lines."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), lc,
				 "Set maximum line length for comment lines."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), lp,
				 "if this option is specified, and a line has a left parenthesis which is not closed on that line, then continuation lines will be lined up to start at the character position just after the left parenthesis.  This processing also applies to `[ and applies to `{ when it occurs in initialization lists."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), lps,
				 "Leave space between `# and preprocessor directive."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), pcs,
				 "This option causes a space to be placed between the name of the procedure being called and the `(."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), prs,
				 "This option causes all parentheses to be seperated with a space from the what is between them."
				 ,""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), psl,
				 "This option causes the type of a procedure being defined to be placed on the line before the name of the procedure. This style is required for the etags program to work correctly, as well as some of the c-mode functions of Emacs."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), saf,
				 "This option forces a space between an for and  the following parenthesis."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), sai,
				 "This option forces a space between an if and the following parenthesis."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), saw,
				 "This option forces a space between an while and the following parenthesis."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), sbi,
				 "Indent braces of a struct, union or enum."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), sc,
				 "This option causes stars to be placed at the beginning of  multi-line  comments."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), sob,
				 "This option causes indent to swallow optional blank lines (that is, any optional blank lines present in the input will be removed from the output)."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), ss,
				 "On one-line for and while statments, force a blank before the semicolon."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), ts,
				 "Indent assumes that tabs are placed at regular intervals of both input and output character streams. These intervals are by default 8 columns wide, but may be changed with this option. Tabs are treated as the equivalent number of spaces."
				 , ""); 
   gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), ut,
				 "Use tabs."
				 , ""); 


  /* Save references to widgets */
  indentdialog->bad = GTK_WIDGET (bad);
  indentdialog->bap = GTK_WIDGET (bap);
  indentdialog->bbb = GTK_WIDGET (bbb);
  indentdialog->bbo = GTK_WIDGET (bbo);
  indentdialog->bl = GTK_WIDGET (bl);
  indentdialog->bc = GTK_WIDGET (bc);
  indentdialog->bli = GTK_WIDGET (bli);
  indentdialog->bls = GTK_WIDGET (bls);
  indentdialog->br = GTK_WIDGET (br);
  indentdialog->brs = GTK_WIDGET (brs);
  indentdialog->bs = GTK_WIDGET (bs);
  indentdialog->c = GTK_WIDGET (c);
  indentdialog->cbi = GTK_WIDGET (cbi);
  indentdialog->cd = GTK_WIDGET (cd);
  indentdialog->cdb = GTK_WIDGET (cdb);
  indentdialog->cdw = GTK_WIDGET (cdw);
  indentdialog->ce = GTK_WIDGET (ce);
  indentdialog->ci = GTK_WIDGET (ci);
  indentdialog->cli = GTK_WIDGET (cli);
  indentdialog->cp = GTK_WIDGET (cp);
  indentdialog->cs = GTK_WIDGET (cs);
  indentdialog->d = GTK_WIDGET (d);
  indentdialog->bfda = GTK_WIDGET (bfda);
  indentdialog->di = GTK_WIDGET (di);
  indentdialog->fc1 = GTK_WIDGET (fc1);
  indentdialog->fca = GTK_WIDGET (fca);
  indentdialog->hnl = GTK_WIDGET (hnl);
  indentdialog->i = GTK_WIDGET (i);
  indentdialog->ip = GTK_WIDGET (ip);
  indentdialog->l = GTK_WIDGET (l);
  indentdialog->lc = GTK_WIDGET (lc);
  indentdialog->lp = GTK_WIDGET (lp);
  indentdialog->lps = GTK_WIDGET (lps);
  indentdialog->pcs = GTK_WIDGET (pcs);
  indentdialog->pi = GTK_WIDGET (pi);
  indentdialog->prs = GTK_WIDGET (prs);
  indentdialog->psl = GTK_WIDGET (psl);
  indentdialog->saf = GTK_WIDGET (saf);
  indentdialog->sai = GTK_WIDGET (sai);
  indentdialog->saw = GTK_WIDGET (saw);
  indentdialog->sbi = GTK_WIDGET (sbi);
  indentdialog->sc = GTK_WIDGET (sc);
  indentdialog->sob = GTK_WIDGET (sob);
  indentdialog->ss = GTK_WIDGET (ss);
  indentdialog->ts = GTK_WIDGET (ts);
  indentdialog->ut = GTK_WIDGET (ut);

  /* Callback connections */
  gtk_signal_connect (GTK_OBJECT (bad), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bap), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bbb), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bbo), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bc), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bl), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bli), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bls), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (br), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (brs), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bs), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (c), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (cbi), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (cd), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (cdb), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (ce), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (cdw), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (ci), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (cli), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (cp), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (cs), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (d), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (bfda), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (hnl), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (di), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (fc1), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (fca), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (i), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (ip), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (l), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (lc), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (lp), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (lps), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (pcs), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (pi), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (prs), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (psl), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (saf), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (sai), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (saw), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (sbi), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (sc), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (sob), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (ss), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (ts), "changed",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
  gtk_signal_connect (GTK_OBJECT (ut), "toggled",
		      GTK_SIGNAL_FUNC (on_custom_option_activated), NULL);
}

static void
indent_create_notebook ()
{
  GtkTooltips *button_bar_tips;
  GtkWidget *notebook1;
  GtkWidget *vbox1;
  GSList *coding_group = NULL;
  GtkWidget *coding_gnu;
  GtkWidget *coding_kr;
  GtkWidget *coding_berkeley;
  GtkWidget *radiobutton1;
  GtkWidget *coding;
  GtkWidget *dialog_vbox;

  dialog_vbox = GNOME_DIALOG (indentdialog->dialog)->vbox;
  gtk_widget_show (dialog_vbox);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_container_add (GTK_CONTAINER (dialog_vbox), notebook1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "vbox1", vbox1,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox1);

  coding_gnu = gtk_radio_button_new_with_label (coding_group, _("GNU"));
  coding_group = gtk_radio_button_group (GTK_RADIO_BUTTON (coding_gnu));
  gtk_widget_ref (coding_gnu);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "coding_gnu",
			    coding_gnu, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (coding_gnu);
  gtk_box_pack_start (GTK_BOX (vbox1), coding_gnu, FALSE, FALSE, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (coding_gnu), TRUE);

  coding_kr =
    gtk_radio_button_new_with_label (coding_group, _("Kernighan & Ritchie"));
  coding_group = gtk_radio_button_group (GTK_RADIO_BUTTON (coding_kr));
  gtk_widget_ref (coding_kr);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "coding_kr",
			    coding_kr, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (coding_kr);
  gtk_box_pack_start (GTK_BOX (vbox1), coding_kr, FALSE, FALSE, 0);

  coding_berkeley =
    gtk_radio_button_new_with_label (coding_group, _("Original Berkeley"));
  coding_group = gtk_radio_button_group (GTK_RADIO_BUTTON (coding_berkeley));
  gtk_widget_ref (coding_berkeley);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog),
			    "coding_berkeley", coding_berkeley,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (coding_berkeley);
  gtk_box_pack_start (GTK_BOX (vbox1), coding_berkeley, FALSE, FALSE, 0);

  radiobutton1 = gtk_radio_button_new_with_label (coding_group, _("Custom"));
  coding_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_widget_ref (radiobutton1);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "radiobutton1",
			    radiobutton1,
			    (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton1);
  indentdialog->custom_radio = radiobutton1;
  gtk_box_pack_start (GTK_BOX (vbox1), radiobutton1, FALSE, FALSE, 0);

  coding = gtk_label_new (_("Coding Style"));
  gtk_widget_ref (coding);
  gtk_object_set_data_full (GTK_OBJECT (indentdialog->dialog), "coding",
			    coding, (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (coding);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
			      gtk_notebook_get_nth_page (GTK_NOTEBOOK
							 (notebook1), 0),
			      coding);
  /* Tooltips */
  button_bar_tips = gtk_tooltips_new();
  gtk_tooltips_set_tip(GTK_TOOLTIPS (button_bar_tips), coding_gnu,
				 "This options selects the option settings that make up the GNU coding style. The GNU coding style is that preferred by the GNU project."
				 ,""); 
  gtk_tooltips_set_tip (GTK_TOOLTIPS (button_bar_tips), coding_kr,
				 "This options selects the option settings that make up the Kernighan & Ritchie coding style. The  Kernighan  &  Ritchie  style is used throughout their well-known book \"The  C  Programming  Language\".","");

  gtk_tooltips_set_tip (GTK_TOOLTIPS (button_bar_tips), coding_berkeley,
				 "This options selects the option settings that make up the original Berkeley style.","");
  gtk_tooltips_set_tip (GTK_TOOLTIPS (button_bar_tips), radiobutton1,
				 "Specify your own coding style options.","");

  create_custom_tabs (notebook1);

  gtk_signal_connect (GTK_OBJECT (coding_gnu), "toggled",
		      GTK_SIGNAL_FUNC (on_coding_gnu_toggled), NULL);
  gtk_signal_connect (GTK_OBJECT (coding_kr), "toggled",
		      GTK_SIGNAL_FUNC (on_coding_kr_toggled), NULL);
  gtk_signal_connect (GTK_OBJECT (coding_berkeley), "toggled",
		      GTK_SIGNAL_FUNC (on_coding_berkeley_toggled), NULL);
  gtk_signal_connect (GTK_OBJECT (radiobutton1), "toggled",
		      GTK_SIGNAL_FUNC (on_coding_custom_toggled), NULL);

}

static void
indent_create_dialog ()
{
  /* Create a OK or Cancel Dialog */
  indentdialog->dialog =
    gnome_dialog_new (_("Indent Options"), GNOME_STOCK_BUTTON_OK,
		      GNOME_STOCK_BUTTON_CANCEL, NULL);
  gtk_window_set_policy (GTK_WINDOW (indentdialog->dialog), FALSE, FALSE,
			 FALSE);

  gtk_widget_show (GNOME_DIALOG (indentdialog->dialog)->vbox);
  /* Add options notebook */
  indent_create_notebook ();
  indent_gnu_set_defaults ();
  /* OK button runs indent */
  gnome_dialog_button_connect (GNOME_DIALOG (indentdialog->dialog), 0,
			       GTK_SIGNAL_FUNC (indent_execute), NULL);
  /* Cancel closes dialog */
  gnome_dialog_button_connect (GNOME_DIALOG (indentdialog->dialog), 1,
			       GTK_SIGNAL_FUNC (indent_close_dialog), NULL);
  gtk_signal_connect (GTK_OBJECT (indentdialog->dialog), "delete_event",
		      GTK_SIGNAL_FUNC (indent_close_dialog), NULL);
  /* Show Dialog */
  gtk_widget_show (indentdialog->dialog);
}


/* 
 * Return text of current document 
 */
GString *
indent_get_document (GideTool * tool)
{
  GString *ret = NULL;

  if (gide_get_document_length (tool) > 0)
    ret =
      g_string_new (gide_get_document_chars
		    (tool, 0, gide_get_document_length (tool)));
  return ret;
}

static void
indent (GtkWidget * widget, gpointer data)
{
  /* Create indentinfo */
  indentinfo = g_new (indent_info, 1);
  /* Create indentdialog */
  indentdialog = g_new (indent_dialog, 1);
  /* Create temp filename */
  indentinfo->temp_file_name = tempnam (g_get_tmp_dir (), "gide-");
  /* Set default coding style */
  indentinfo->coding_style = STYLE_GNU;
  /* Safe reference to tool */
  indentinfo->tool = GIDE_TOOL (data);
  indentinfo->setting_default = FALSE;
  /* Create and show dialog */
  indent_create_dialog ();
}

/*
 * Define the verbs in this plugin
 */
static BonoboUIVerb verbs[] = {
  BONOBO_UI_UNSAFE_VERB ("Indent", indent),
  BONOBO_UI_VERB_END
};

/*
 * Boilerplate initialization function
 */
static gboolean
impl_init (GideTool * tool, gpointer closure)
{
  BonoboUIComponent *uic;
  CORBA_Environment ev;

  g_return_val_if_fail (tool != NULL, FALSE);
  g_return_val_if_fail (GIDE_IS_TOOL (tool), FALSE);

  CORBA_exception_init (&ev);

  uic = bonobo_ui_component_new ("gide-indent-plugin");
  bonobo_ui_component_set_container (uic, tool->ui_container);
  bonobo_ui_util_set_ui (uic, GIDE_DATADIR,
			 "gide-indent-plugin.xml", "gide-indent-plugin");

  bonobo_ui_component_add_verb_list_with_data (uic, verbs, tool);

  CORBA_exception_free (&ev);

  return TRUE;
}

/*
 * Boilerplace cleanup function
 */
static void
impl_cleanup (GideTool * tool, gpointer closure)
{
  bonobo_ui_component_unset_container (gtk_object_get_data
				       (GTK_OBJECT (tool), "ui_component"));
  gtk_object_remove_data (GTK_OBJECT (tool), "ui_component");
}

/*
 * Boilerplate creation function
 */
static CORBA_Object
make_object (PortableServer_POA poa, const char *iid, gpointer impl_ptr,
	     CORBA_Environment * ev)
{
  GideTool *tool;

  tool = gide_tool_new (impl_init, impl_cleanup, NULL);

  oaf_plugin_use (poa, impl_ptr);

  return BONOBO_OBJREF (tool->props);
}

/*
 * List of plugins
 */
static const OAFPluginObject plugin_list[] = {
  {
   SAMPLE_COMPONENT_IID,
   make_object},
  {
   NULL}
};

/*
 * Plugin description
 */
const OAFPlugin OAF_Plugin_info = {
  plugin_list,
  "gIDE Indent Plugin"
};
